/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter;

import java.util.List;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Handler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * abstrakte Implementierung eines Handlers der "Umfassenden Datenanalyse".<br>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class AbstractUdaHandler extends Handler {

	/**
	 * prüft die Ausführbarkeit des Handlers.<br>
	 * Wenn der Handler mit den gegebenem Operator und den Operanden nicht
	 * ausführbar ist wird ein ensprechender Fehler erzeugt (Falsche
	 * Parameteranzahl oder falsche Operandentypen).
	 *
	 * @param operator
	 *            der zu verwendende Operator
	 * @param operanden
	 *            die Operanden
	 */
	protected final void erzeugeHandlerFehler(final Operator operator, final List<?> operanden) {
		final HandlerValidation validation = validiereHandler(operator, operanden);
		if (!validation.isValid()) {
			if (!validation.isRichtigeAnzahl()) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ANZAHL);
			}
			if (!validation.isRichtigerTyp()) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}
			throw new ArgumentFehler(UdaFehlerSubtyp.UNBEKANNT);
		}
	}

	/**
	 * ermittelt, ob die übergebenen Operanden den übergebenen Typen
	 * entsprechen.
	 *
	 * @param operanden
	 *            die Operanden
	 * @param types
	 *            die erwarteten Typen
	 * @return <i>true</i>, wenn die Operanden den Typen entsprechen
	 */
	protected final boolean isAusfuehrbar(final List<?> operanden,
			@SuppressWarnings("unchecked") final Class<? extends Object>... types) {
		boolean result = true;
		if (operanden.size() == types.length) {
			for (int idx = 0; idx < operanden.size(); idx++) {
				final Object obj = operanden.get(idx);
				if (!types[idx].getClass().isInstance(obj)) {
					result = false;
					break;
				}
			}
		} else {
			result = false;
		}
		return result;
	}

	/**
	 * ermittelt ob der über den Index identifizierte Operand aus der
	 * übergebenen Operandenliste einem der übergebenen Typen entspricht.
	 *
	 * @param operanden
	 *            die Operandenliste
	 * @param offset
	 *            der Index des zu betrachtenden Operanden
	 * @param types
	 *            die möglichen Typen
	 * @return <i>true</i>, wenn der Operand den Erwartungen entspricht
	 */
	protected final boolean isValidOperand(final List<?> operanden, final int offset,
			final Class<?>... types) {
		boolean result = false;
		if (operanden.size() > offset) {
			final Object obj = operanden.get(offset);
			for (final Class<?> type : types) {
				if (type.isInstance(obj)) {
					// if (type.getClass().isInstance(obj)) {
					result = true;
					break;
				}
			}
		}
		return result;
	}

}
