/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import de.bsvrz.ibv.uda.interpreter.UdaInterpreter;
import de.bsvrz.ibv.uda.interpreter.ausdruck.ElementZugriffsSymbol;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Variable;
import de.bsvrz.ibv.uda.interpreter.daten.container.ContainerZugriffsSymbol;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.SymbolUndefiniertFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.SymbolUndefiniertException;

/**
 * die Repräsentation einer Zuweisungsanweisung in einem Uda-Skript.<br>
 * Eine Zuweisungsanweisung wertet den ihr zugeordneten Ausdruck und weist das
 * Ergebnis einem Zeilausdruck zu. Der Zielausdruck kann entweder eine Variable
 * im aktuellen Kontext, ein ELement einer Datenstruktur oder ein Element eines
 * Containers sein.<br>
 * Zur Erhöhung der Zuverlässigkeit und Typsicherheit werden nicht alle
 * Zuweisungen erlaubt. Innerhalb einer Zuweisungsanweisung sind für die
 * Zuweisung auf Variablen folgende Zuweisungen möglich:
 * <ul>
 * <li>die Zuweisung eines Objekt zu einem Ziel gleichen Typs</li>
 * <li>die Zuweisung eines Objekts zum einer Ziel mit dem Wert "undefiniert"
 * </li>
 * <li>die Zuweisung eines Zahlenwertes zu einem Ziel mit einem Zahlenwert</li>
 * <li>die Zuweisung eines Objekts zu einem Ziel mit dem Typ "Zeichenkette"</li>
 * <li>die Zuweisung des Wertes "undefiniert" zu jedem beliebigen Ziel</li>
 * </ul>
 * Alle anderen Zuweisungen führen zu einer Ausnahme. Über die oben genannten
 * Regeln für Zuweisungen können Zuweisungen zu bestimmten Objekten zusätzliche
 * Einschränkungen besitzen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZuweisungsAnweisung extends AbstractEinfacheAnweisung {

	/**
	 * der Ausdruck, der das Ziel der Zuweisung beschreibt.
	 */
	private final Ausdruck destination;

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der die Anweisung definiert ist
	 * @param skriptname
	 *            der Name des Skripts
	 * @param destination
	 *            der Ausdruck für das Zuweisungsziel
	 * @param ausdruck
	 *            der auszuwertende Ausdruck, der den zuzuweisenden Wert liefert
	 */
	public ZuweisungsAnweisung(final int zeile, final String skriptname, final Ausdruck destination,
			final Ausdruck ausdruck) {
		super(zeile, skriptname, ausdruck);
		this.destination = destination;
	}

	/**
	 * führt die Anwiesung aus.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		final Ausdruck ausdruck = getAusdruck();
		Object daten = null;
		if (ausdruck != null) {

			try {
				daten = ausdruck.interpret(kontext);
				if (daten instanceof UdaFehler) {
					daten = erzeugeAnweisungsFehler((UdaFehler) daten);
				}
			} catch (final InterpreterException e) {
				daten = erzeugeAnweisungsFehler(e);
			}

			if (daten == null) {
				daten = NichtWert.NICHTWERT;
			}

			try {
				if (destination instanceof Variable) {
					final Variable variable = (Variable) destination;
					Object setData = null;
					Object destData = null;
					try {
						destData = kontext.get(variable.getName());
					} catch (@SuppressWarnings("unused") final SymbolUndefiniertException e) {
						// Variable existiert im Kontext noch nicht
					}
					if (destData instanceof SymbolUndefiniertFehler) {
						destData = null;
					}
					setData = UdaInterpreter.getZuweisbareDaten(destData, daten);
					if (setData == null) {
						if (destData == null) {
							destData = NichtWert.NICHTWERT;
						}
						final UdaFehler fehler = new ObjektZugriffsFehler(UdaFehlerSubtyp.VARIABLE,
								"Der Wert vom Typ \""
										+ daten.getClass().getSimpleName() + "\" kann der Variable \""
										+ variable.getName() + "\" mit dem Typ \""
										+ destData.getClass().getSimpleName() + "\" nicht zugewiesen werden");
						fehler.setSkript(getSkript());
						fehler.setZeile(getZeile());
						throw fehler;
					}
					kontext.set(variable.getName(), setData);
				} else if (destination instanceof ElementZugriffsSymbol) {
					final ElementZugriffsSymbol symbol = (ElementZugriffsSymbol) destination;
					symbol.set(kontext, daten);
				} else if (destination instanceof ContainerZugriffsSymbol) {
					final ContainerZugriffsSymbol container = (ContainerZugriffsSymbol) destination;
					container.set(kontext, daten);
				}
			} catch (final InterpreterException e) {
				throw erzeugeAnweisungsFehler(e);
			}
		}

		return daten;
	}

}
