/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.ausdruck;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.SymbolUndefiniertFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.ibv.uda.interpreter.daten.java.JavaObjekt;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerNotFoundException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * die Repräsentation eines Ausdrucks zur Beschreibung eines Elementzugriffs.
 * <br>
 * Der Elementzugriff kann ein Attributzugriff oder eine Memberfunktion eines
 * Objekts sein und wird in Uda durch den Punkt-Operator dargestellt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ElementZugriffsSymbol implements Ausdruck {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * der Ausdruck, der das Objekt, auf dessen Element zugegriffen werden soll
	 * beschreibt.
	 */
	private final Ausdruck objektAusdruck;

	/**
	 * der Ausdruck, der den Zugriff beschreibt (Attribut oder Funktionsaufruf).
	 */
	private final Ausdruck zugriffsAusdruck;

	/**
	 * Konstruktor.<br>
	 *
	 * @param objectAusdruck
	 *            der Ausdruck, der das Objekt beschreibt, auf das zugegriffen
	 *            werden soll
	 * @param zugriffsAusdruck
	 *            der Ausdruck, der beschreibt, wie auf das Objekt zugegriffen
	 *            werden soll
	 */
	public ElementZugriffsSymbol(final Ausdruck objectAusdruck, final Ausdruck zugriffsAusdruck) {
		this.objektAusdruck = objectAusdruck;
		this.zugriffsAusdruck = zugriffsAusdruck;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object result = null;
		String name = null;
		final Object object = objektAusdruck.interpret(kontext);
		if (zugriffsAusdruck instanceof Variable) {
			name = ((Variable) zugriffsAusdruck).getName();
			if ("definiert".equals(name)) {
				if (object == null) {
					result = LogischerWert.FALSCH;
				} else {
					if (NichtWert.NICHTWERT.equals(object)) {
						result = LogischerWert.FALSCH;
					} else {
						result = LogischerWert.WAHR;
					}
				}
			} else if (object instanceof Struktur) {
				result = ((Struktur) object).getStrukturElement(name);
			}

		} else if (zugriffsAusdruck instanceof FunktionsAufruf) {
			final FunktionsAufruf funktion = (FunktionsAufruf) zugriffsAusdruck;
			name = funktion.getName();
			if (object instanceof JavaObjekt) {
				return ((JavaObjekt) object).execute(kontext, name, funktion.getArgumente());
			}

			final List<Ausdruck> operanden = new ArrayList<>();
			operanden.add(objektAusdruck);
			operanden.addAll(funktion.getArgumente());
			if (Operator.enthaelt(name)) {
				try {
					result = new Operation(Operator.getOperator(name), operanden).interpret(kontext);
				} catch (final HandlerNotFoundException e) {
					final String message = "Es gibt keine Methode mit dem Namen: " + name
							+ " für das Objekt vom Typ: "
							+ object.getClass().getName();
					LOGGER.error(message, e);
					throw new ObjektZugriffsFehler(UdaFehlerSubtyp.FUNKTION,
							message);
				}
			} else {
				throw new ObjektZugriffsFehler(UdaFehlerSubtyp.FUNKTION,
						"Es gibt keine Methode mit dem Namen: " + name);
			}
		}

		if (result == null) {
			throw new SymbolUndefiniertFehler(UdaFehlerSubtyp.ELEMENT);
		}
		return result;
	}

	/**
	 * setzt den Wert des Objekts, des durch den {@link #objektAusdruck}
	 * beschriebenen Objekts auf den übergebenen Wert.
	 *
	 * @param kontext
	 *            der Kontext für die Ausführung der Operation
	 * @param daten
	 *            die Daten, die gesetzt werden sollen
	 */
	public void set(final Kontext kontext, final Object daten) {
		if (zugriffsAusdruck instanceof Variable) {
			final Object object = objektAusdruck.interpret(kontext);
			final Struktur struktur = (Struktur) object;
			struktur.setStrukturElement(((Variable) zugriffsAusdruck).getName(), daten);
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(objektAusdruck);
		liste.add(zugriffsAusdruck);
		return liste;
	}

}
