/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.ausdruck;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import de.bsvrz.ibv.uda.interpreter.AnweisungsBlock;
import de.bsvrz.ibv.uda.interpreter.anweisung.RueckgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.container.Feld;
import de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer;
import de.bsvrz.ibv.uda.interpreter.daten.dav.MengenObjekt;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.ZeichenkettenLiteral;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * ein Ausdruck zu Darstellung eines Quantoraufrufes innerhalb einer
 * Uda-Skript-Anweisung.<br>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class Quantor implements Ausdruck {

	/**
	 * der Name der Laufvariable, die während der Iteration über die
	 * auszuwertende Menge di einzelnen Elemente enthält.
	 */
	private final String variable;

	/**
	 * die zu prüfende Quantität.
	 */
	private final QuantorBedingung quantitaet;

	/**
	 * der Ausdruck, der die Menge, die zu überprüfen ist beschreibt.
	 */
	private final Ausdruck mengenAusdruck;

	/**
	 * die Bedingung, gegen die die Elemente der Menge geprüft werden sollen.
	 */
	private final Ausdruck bedingung;

	/**
	 * Konstruktor.<br>
	 *
	 * @param variable
	 *            die Laufvariable für die Auswertung der Menge
	 * @param quantitaet
	 *            die Quantor-Bedingung
	 * @param menge
	 *            die Menge über die die Auswertung erfolgen soll
	 * @param bedingung
	 *            die Bedingung, die für alle Elemente der Menge geprüft werden
	 *            soll
	 */
	public Quantor(final Variable variable, final QuantorBedingung quantitaet, final Ausdruck menge,
			final Ausdruck bedingung) {
		this.variable = variable.getName();
		this.quantitaet = quantitaet;
		mengenAusdruck = menge;
		this.bedingung = bedingung;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(new ZeichenkettenLiteral(variable));
		liste.add(quantitaet);
		liste.add(mengenAusdruck);
		liste.add(bedingung);
		return liste;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		final SkriptKontext lokalerKontext = new SkriptKontext((SkriptKontext) kontext, true);
		final Object mengenObject = mengenAusdruck.interpret(lokalerKontext);
		UdaContainer menge = null;
		if (mengenObject instanceof UdaContainer) {
			menge = (UdaContainer) mengenObject;
		} else if (mengenObject instanceof MengenObjekt) {
			menge = (UdaContainer) ((MengenObjekt) mengenObject).getStrukturElement("objekte");
		}

		if (menge == null) {
			throw new Fehler("Der Quantorausdruck hat keine gültige Menge");
		}

		final UdaContainer ergebnisse = new Feld();
		int wahr = 0;
		int falsch = 0;
		for (final Object element : menge.getElemente()) {
			lokalerKontext.set(variable, element);
			final Object check = bedingung.interpret(lokalerKontext);
			if (check instanceof LogischerWert) {
				final boolean wert = ((LogischerWert) check).get();
				ergebnisse.einfuegen(0, check);
				if (wert) {
					wahr++;
				} else {
					falsch++;
				}
			} else {
				throw new Fehler("Die Auswertung eines Elements ergab keinen logischen Wert");
			}
		}

		if (quantitaet.getTyp() == QuantorBedingung.NUTZER) {
			Object result = null;
			final QuantorDefinition qDef = ((SkriptKontext) kontext).getInterpreter().getQuantor(quantitaet.getName());

			final AnweisungsBlock block = new AnweisungsBlock(0, "");
			block.addAnweisung(new RueckgabeAnweisung(0, "", bedingung));

			final FunktionsDefinition funktion = new FunktionsDefinition(0, "", qDef.getBedingungsName(),
					Collections.singletonList(variable), block);
			final Set<String> variablen = Ausdruck.getVariablen(block);
			for (final String v : variablen) {
				if (!variable.equals(v)) {
					funktion.setKonstante(v, kontext.get(v));
				}
			}

			final SkriptKontext quantorKontext = new SkriptKontext((SkriptKontext) kontext, true);
			quantorKontext.setFunktion(qDef.getBedingungsName(), funktion);
			quantorKontext.set(qDef.getMengenName(), mengenObject);
			final Object auswertungsErgebnis = qDef.interpret(quantorKontext);
			if (auswertungsErgebnis instanceof LogischerWert) {
				result = auswertungsErgebnis;
			} else {
				result = new Fehler("Die Auswertung des Quantors liefert keinen logischen Wert");
			}

			return result;
		}
		return quantitaet.check(ergebnisse, wahr, falsch, lokalerKontext);
	}
}
