/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.hysterese;

import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.*;

/**
 * Handler für Operationen mit Hysterese-Objekten innerhalb eines
 * Uda-Skript-Interpreters.<br>
 * Unterstützte Operatoren sind:
 * <ul>
 * <li><b>Hysterese</b> - erzeugt ein Hystereseobjekt</li>
 * <li><b>hystereseStufe</b> - erzeugt eine Hysteresestufe</li>
 * <li><b>stufe</b> - ermittelt die aktuelle Stufe einer Hysterese</li>
 * <li><b>klassifiziere</b> - klassifiziert einen Wert und setzt die aktuelle
 * Stufe eines Hystereseobjekts</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class HystereseHandler extends AbstractUdaHandler {
	/**
	 * {@inheritDoc}.
	 *
	 * @return die Liste der verfügbaren Operatoren
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return HystereseOperators.OPERATOREN;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<?> operanden) {
		boolean typOk = false;
		boolean anzahlOk = false;

		if (HystereseOperators.HYSTERESE_KONSTRUKTOR.equals(operator)) {
			anzahlOk = true;
			typOk = true;
			for (final Object operand : operanden) {
				if (!(operand instanceof HystereseStufe)) {
					typOk = false;
					break;
				}
			}
		} else if (HystereseOperators.STUFEN_KONSTRUKTOR.equals(operator)) {
			if (operanden.size() == 3) {
				anzahlOk = true;
				if (Handler.getOperand(operanden, 0) instanceof String) {
					if (Handler.getOperand(operanden, 1) instanceof Number) {
						if (Handler.getOperand(operanden, 2) instanceof Number) {
							typOk = true;
						}
					}
				}
			}
		} else if (HystereseOperators.STUFE.equals(operator)) {
			if (operanden.size() <= 2) {
				anzahlOk = true;
				if (Handler.getOperand(operanden, 0) instanceof Hysterese) {
					final Object operand = Handler.getOperand(operanden, 1);
					if ((operand == null) || (operand instanceof String) || (operand instanceof HystereseStufe)) {
						typOk = true;
					}
				}
			}
		} else if (HystereseOperators.KLASSIFIZIERE.equals(operator)) {
			if (operanden.size() == 2) {
				anzahlOk = true;
				if (Handler.getOperand(operanden, 0) instanceof Hysterese) {
					if (Handler.getOperand(operanden, 1) instanceof Number) {
						typOk = true;
					}
				}
			}
		}

		return new HandlerValidation(anzahlOk, typOk);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<?> operanden) {
		erzeugeHandlerFehler(operator, operanden);
		Object result = null;

		try {
			if (operator.equals(HystereseOperators.HYSTERESE_KONSTRUKTOR)) {
				result = erzeugeHysterese(operanden);
			} else if (operator.equals(HystereseOperators.STUFEN_KONSTRUKTOR)) {
				result = erzeugeHystereseStufe(operanden);
			} else if (operator.equals(HystereseOperators.STUFE)) {
				final Hysterese hysterese = (Hysterese) operanden.get(0);
				if (operanden.size() == 1) {
					result = hysterese.getStufe();
				} else {
					hysterese.setStufe(operanden.get(1));
					result = hysterese.getStufe();
				}
			} else if (operator.equals(HystereseOperators.KLASSIFIZIERE)) {
				final Hysterese hysterese = (Hysterese) operanden.get(0);
				final Number wert = (Number) operanden.get(1);
				result = hysterese.klassifiziere(wert);
			}
		} catch (@SuppressWarnings("unused") final ClassCastException e) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
		}
		return result;
	}

	/**
	 * erzeugt eine neue Hysteresestufe und initialisiert diese mit den
	 * übergebenen Parametern.
	 *
	 * @param operanden
	 *            die Argumente
	 * @return die Hysteresestufe
	 */
	private HystereseStufe erzeugeHystereseStufe(final List<?> operanden) {
		final String stufe = (String) operanden.get(0);
		final Number anfang = (Number) operanden.get(1);
		final Number ende = (Number) operanden.get(2);
		return new HystereseStufe(stufe, anfang, ende);
	}

	/**
	 * erzeugt ein Hysteresobjekt und füügt die übergebene Liste von
	 * Hysteresestufen in das Objekt ein.
	 *
	 * @param stufen
	 *            die Stufen die das Hystereobjekt initial enthalten soll
	 * @return die erzeugte Hystereseobjekt
	 */
	private Hysterese erzeugeHysterese(final List<?> stufen) {
		return new Hysterese(stufen.toArray(new HystereseStufe[0]));
	}
}
