/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.client;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bitctrl.Constants;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.uda.data.Historie;
import de.bsvrz.ibv.uda.uda.data.LaufKommandoAntwort;
import de.bsvrz.ibv.uda.uda.data.SkriptLauf;
import de.bsvrz.ibv.uda.uda.data.SkriptLaufKommando;
import de.bsvrz.ibv.uda.uda.data.SkriptLaufStatus;
import de.bsvrz.ibv.uda.uda.data.StartTyp;
import de.bsvrz.ibv.uda.uda.data.UDA;
import de.bsvrz.ibv.uda.verwaltung.protokoll.DateiProtokoll;
import de.bsvrz.ibv.uda.verwaltung.protokoll.DavProtokoll;
import de.bsvrz.ibv.uda.verwaltung.protokoll.KonsoleProtokoll;
import de.bsvrz.ibv.uda.verwaltung.protokoll.LogProtokoll;
import de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll;
import de.bsvrz.ibv.uda.verwaltung.protokoll.ProtokollEintrag;
import de.bsvrz.ibv.uda.verwaltung.protokoll.ProtokollTyp;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * die Repräsentation eines Skriptlaufs innerhalb eines Clients der "Umfassenden
 * Datenanalyse".
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ClientSkriptLauf extends SkriptLauf implements ClientReceiverInterface {

	/**
	 * Interne Klasse zum Warten auf Antworten für Kommmandos zur
	 * Skriptlaufausführung.
	 *
	 * @author peuker
	 */
	private static class KommandoAntwortBeobachter {

		/**
		 * die Liste der Anfragen mit ihren Antworten.
		 */
		private final Map<Long, LaufKommandoAntwort> antworten = new HashMap<>();

		/**
		 * der interne Zähler für die Vergabe von ID für Anfragen.
		 */
		private int zaehler = (int) (System.currentTimeMillis() / 1000L);

		/**
		 * die Id der Anwendung innerhalb der Datenverteilers.
		 */
		private final long appId;

		/** die verwendete Datenverteilerverbindung. */
		private final ClientDavInterface verbindung;

		/**
		 * Konstruktor.
		 *
		 * @param verbindung
		 *            die verwendete Datenverteilerverbindung
		 */
		KommandoAntwortBeobachter(final ClientDavInterface verbindung) {
			this.verbindung = verbindung;
			this.appId = verbindung.getLocalApplicationObject().getId();
		}

		/**
		 * verarbeitet eine enpfangene Antwort und erzeugt das Antwortobjekt.
		 *
		 * @param daten
		 *            die empfamgenen Daten
		 */
		public void antwort(final Data daten) {
			if (daten != null) {
				final long anwendung = daten.getUnscaledValue("Anwendung")
						.longValue();
				if (anwendung == appId) {
					final long id = daten.getUnscaledValue("Id").longValue();
					final SkriptLaufKommando typ = SkriptLaufKommando.getZustand(daten.getUnscaledValue("Kommando")
							.intValue());

					String meldung = Constants.EMPTY_STRING;
					if (daten.getTextArray("Meldung").getLength() > 0) {
						meldung = daten.getTextArray("Meldung").getText(0);
					}
					synchronized (antworten) {
						antworten.put(id, new LaufKommandoAntwort(typ, meldung));
					}
				}
			}
		}

		/**
		 * liefert die ID der Applikation für die Antworten verarbeitet werden.
		 *
		 * @return die Id
		 */
		public long getAppId() {
			return appId;
		}

		/**
		 * liefert die ID für eine neue Anfrage und trägt diese in die Liste der
		 * erwarteten Rückmeldungen ein.
		 *
		 * @return die Id
		 */
		public long getId() {
			zaehler++;
			final long id = (Long.valueOf(hashCode()) << 32) | zaehler;
			antworten.put(id, null);
			return id;
		}

		/**
		 * liest die Antwort für die übergebene Anfrage-Id aus.<br>
		 * Wurde innerhalb der angegebenen Zeit keine Antwort empfangen, wird
		 * eine Exception geworfen.
		 *
		 * @param id
		 *            die Id der Anfrage
		 * @param timeout
		 *            die maximale Wartezeit in Millisekunden
		 * @return die Antwort
		 * @throws ClientException
		 *             es konnte keine Antwort ermittelt werden
		 */
		public LaufKommandoAntwort liesAntwort(final long id, final long timeout) throws ClientException {
			final long endZeit = System.currentTimeMillis() + timeout;
			LaufKommandoAntwort antwort = null;
			do {
				synchronized (antworten) {
					antwort = antworten.get(id);
				}
				if (antwort == null) {
					if (!antworten.containsKey(id)) {
						throw new ClientException("Die erwartete Antwort ist nicht als Anfrage registriert.");
					}
					if (System.currentTimeMillis() > endZeit) {
						antworten.remove(id);
						throw new ClientException(
								"Die erwartete Antwort wurde nicht in der erwarteten Zeit empfangen.");
					}
				}
				verbindung.sleep(200);

			} while (antwort == null);

			antworten.remove(id);
			return antwort;
		}
	}

	/**
	 * das Modul, in dem der Skriptlauf verwaltet wird.
	 */
	private final ClientModul modul;

	/**
	 * das Systemobjekt, das den Skriptlauf innerhalb der Konfiguration des
	 * Datenverteilers repräsentiert.
	 */
	private final SystemObject objekt;

	/**
	 * die Datenbeschreibung zum Versand von Vorgabedaten zur Änderung des
	 * Inhalts der Definition des Skriptlaufs.
	 */
	private final DataDescription vorgabeDesc;

	/**
	 * die Datenbeschreibung zum Empfang der aktuellen Definition des
	 * Skriptlaufes.
	 */
	private final DataDescription istDesc;

	/**
	 * Marker, der die Gültigkeit der Daten des Objekts anzeigt.
	 */
	private boolean valid;

	/**
	 * die Datenbeschreibung zum Empfang des aktuellen Ausführungsstatus des
	 * Skriptlaufs.
	 */
	private final DataDescription statusDesc;

	/**
	 * die Datenbeschreibung zum Empfang von Antworten auf Kommandos zur
	 * Änderung des Skriptlauf-Ausführungsstatus.
	 */
	private final DataDescription kommandoRespDesc;

	/**
	 * die Datenbeschreibung zum Versand von Kommandos zur Änderung des
	 * Skriptlauf-Ausführungsstatus.
	 */
	private final DataDescription kommandoDesc;

	/**
	 * die Datenbeschreibung zum Empfang von Ausgabeinformationen eines aktiven
	 * Skriptlaufs..
	 */
	private final DataDescription logDesc;

	/**
	 * die Datenbeschreibung zum Empfang des Ergebnisses eines Skriptlaufs.
	 */
	private final DataDescription resultDesc;

	/**
	 * die Klasse zur Beobachtung von Antworten auf versendete Kommandos zur
	 * Skriptausführung.
	 */
	private final KommandoAntwortBeobachter kommandoAntwortBeobachter;

	/**
	 * die Liste der Skriptlaufausgaben.
	 */
	private final List<String> meldungen = new ArrayList<>();

	/**
	 * das beim letzten Skriptlauf erhaltene Ergebnis.
	 */
	private SkriptLaufErgebnis letztesErgebnis;

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt ein Skriptlaufobjekt für das übergebene Modul und
	 * das definierte Systemobjekt.
	 *
	 * @param modul
	 *            das UdaModul
	 * @param obj
	 *            das Systemobjekt
	 */
	ClientSkriptLauf(final ClientModul modul, final SystemObject obj) {
		this.modul = modul;
		this.objekt = obj;

		kommandoAntwortBeobachter = new KommandoAntwortBeobachter(modul.getVerbindung());

		final DataModel model = modul.getVerbindung().getDataModel();
		istDesc = new DataDescription(model.getAttributeGroup(UDA.LAUF_ATG_LAUF), model.getAspect(UDA.ASP_IST));
		modul.getVerbindung().subscribeReceiver(this, objekt, istDesc, ReceiveOptions.normal(),
				ReceiverRole.receiver());

		vorgabeDesc = new DataDescription(model.getAttributeGroup(UDA.LAUF_ATG_LAUF), model.getAspect(UDA.ASP_VORGABE));
		modul.getSender().anmelden(objekt, vorgabeDesc, false);

		kommandoDesc = new DataDescription(model.getAttributeGroup(UDA.LAUF_ATG_LAUF_KOMMANDO),
				model.getAspect(UDA.ASP_ANFORDERUNG));
		modul.getSender().anmelden(objekt, kommandoDesc, false);

		kommandoRespDesc = new DataDescription(model.getAttributeGroup(UDA.LAUF_ATG_LAUF_KOMMANDO),
				model.getAspect(UDA.ASP_ANTWORT));
		modul.getVerbindung().subscribeReceiver(this, objekt, kommandoRespDesc, ReceiveOptions.normal(),
				ReceiverRole.receiver());

		statusDesc = new DataDescription(model.getAttributeGroup(UDA.LAUF_ATG_LAUF_ZUSTAND),
				model.getAspect(UDA.ASP_IST));
		modul.getVerbindung().subscribeReceiver(this, objekt, statusDesc, ReceiveOptions.normal(),
				ReceiverRole.receiver());

		logDesc = new DataDescription(model.getAttributeGroup(UDA.LAUF_ATG_LAUF_LOGGER), model.getAspect(UDA.ASP_IST));
		modul.getVerbindung().subscribeReceiver(this, objekt, logDesc, ReceiveOptions.normal(),
				ReceiverRole.receiver());

		resultDesc = new DataDescription(model.getAttributeGroup(UDA.LAUF_ATG_LAUF_ERGEBNIS),
				model.getAspect(UDA.ASP_IST));
		modul.getVerbindung().subscribeReceiver(this, objekt, resultDesc, ReceiveOptions.normal(),
				ReceiverRole.receiver());
	}

	/**
	 * überträgt die daten des übergebenen Datensatzes in die internen
	 * Datenelemente des Objekts.
	 *
	 * @param daten
	 *            die Daten
	 */
	private void aktualisiereDaten(final Data daten) {
		setName(daten.getTextValue(UDA.LAUF_ATT_NAME).getText());
		setBeschreibung(daten.getTextValue(UDA.LAUF_ATT_BESCHREIBUNG).getText());

		final SystemObject skriptObjekt = daten.getReferenceValue(UDA.LAUF_ATT_SKRIPT).getSystemObject();

		if (skriptObjekt == null) {
			setSkript(null);
		} else {
			setSkript(modul.getSkript(skriptObjekt.getPid()));
		}

		clearHistorie();
		final Data.Array historieArray = daten.getArray(UDA.LAUF_ATT_HISTORIE);
		for (int idx = 0; idx < historieArray.getLength(); idx++) {
			final Data historieData = historieArray.getItem(idx);
			final long zeitPunkt = historieData.getTimeValue(UDA.ATL_HISTORIE_ATT_AENDERUNGSDATUM).getMillis();
			final String name = historieData.getTextValue(UDA.ATL_HISTORIE_ATT_NAME).getText();
			final String grund = historieData.getTextValue(UDA.ATL_HISTORIE_ATT_GRUND).getText();

			addHistorie(new Historie(zeitPunkt, name, grund));
		}

		clearParameter();
		final Data.Array parameter = daten.getArray(UDA.LAUF_ATT_PARAMETER);
		clearParameter();
		for (int idx = 0; idx < parameter.getLength(); idx++) {
			final Data eintrag = parameter.getItem(idx);
			final String name = eintrag.getTextValue(UDA.ATL_PARAMETER_ATT_NAME).getText();
			final String wert = eintrag.getTextValue(UDA.ATL_PARAMETER_ATT_WERT).getText();
			setParameter(name, wert);
		}

		setBenutzer(daten.getTextValue(UDA.LAUF_ATT_BENUTZER).getText());

		setStartTyp(StartTyp.getZustand(daten.getUnscaledValue(UDA.LAUF_ATT_STARTTYP).intValue()));

		getZyklus().setStartZeitPunkt(daten.getItem(UDA.LAUF_ATT_STARTZYKLUS)
				.getTimeValue(UDA.ATL_STARTZYKLUS_ATT_STARTZEITPUNKT).getMillis());
		getZyklus().setIntervall(daten.getItem(UDA.LAUF_ATT_STARTZYKLUS)
				.getUnscaledValue(UDA.ATL_STARTZYKLUS_ATT_INTERVALL).longValue());

		final Data.Array protokollArray = daten.getArray(UDA.LAUF_ATT_PROTOKOLL);
		clearProtokolle();
		for (int idx = 0; idx < protokollArray.getLength(); idx++) {
			final Data eintrag = protokollArray.getItem(idx);
			Protokoll protokoll = null;
			final int typ = eintrag.getUnscaledValue(UDA.ATL_PROTOKOLL_ATT_TYP).intValue();
			if (typ == ProtokollTyp.DATEI.getCode()) {
				protokoll = new DateiProtokoll();
			} else if (typ == ProtokollTyp.DAV.getCode()) {
				protokoll = new DavProtokoll();
			} else if (typ == ProtokollTyp.KONSOLE.getCode()) {
				protokoll = new KonsoleProtokoll();
			} else if (typ == ProtokollTyp.LOGGING.getCode()) {
				protokoll = new LogProtokoll();
			}

			if (protokoll != null) {
				protokoll.setName(eintrag.getTextValue(UDA.ATL_PROTOKOLL_ATT_NAME).getText());
				protokoll.setAnzahl(eintrag.getUnscaledValue(UDA.ATL_PROTOKOLL_ATT_ANZAHL).longValue());
				protokoll.setMaxSize(eintrag.getUnscaledValue(UDA.ATL_PROTOKOLL_ATT_MAXGROESSE).longValue());
				addProtokoll(protokoll);
			}
		}
		valid = true;
	}

	/**
	 * Es wird ein Kommando zum Anhalten eines aktiven Skriptlaufs an die
	 * "Umfassende Datenanalyse" übertragen.
	 *
	 * @return das Ergebnis des Kommandoausführung
	 *
	 * @throws ClientException
	 *             der Skriptlauf konnte nicht angehalten werden
	 */
	public LaufKommandoAntwort anhalten() throws ClientException {
		return sendeKommando(SkriptLaufKommando.ANHALTEN);
	}

	/**
	 * Es wird ein Kommando zum Anhalten eines beenden eines Skriptlaufs an die
	 * "Umfassende Datenanalyse" übertragen.
	 *
	 * @return das Ergebnis des Kommandoausführung
	 *
	 * @throws ClientException
	 *             der Skriptlauf konnte nicht beendet werden
	 */
	public LaufKommandoAntwort beenden() throws ClientException {
		return sendeKommando(SkriptLaufKommando.BEENDEN);
	}

	/**
	 * Es wird ein Kommando zum Fortsetzen eines zuvor angehaltenen aktiven
	 * Skriptlaufs an die "Umfassende Datenanalyse" übertragen.
	 *
	 * @return das Ergebnis des Kommandoausführung
	 *
	 * @throws ClientException
	 *             der Skriptlauf konnte nicht fortgesetzt werden
	 */
	public LaufKommandoAntwort fortsetzen() throws ClientException {
		return sendeKommando(SkriptLaufKommando.FORTSETZEN);
	}

	/**
	 * erzeugt und füllt einen Datensatz für die Vorgabe von Definitionsdaten
	 * für den Skriptlauf.
	 *
	 * @param benutzer
	 *            der benutzer der eine Änderung vornimmt
	 * @param grund
	 *            der Grund für die Änderung
	 *
	 * @return einen Datensatz für die Beschreibung des Skriptlaufs
	 * @throws ClientException
	 *             der Datensatz konnte nicht korrekt erstellt bzw. gefüllt
	 *             werden
	 */
	private Data fuelleDatensatz(final String benutzer, final String grund) throws ClientException {
		final Data data = modul.getVerbindung().createData(istDesc.getAttributeGroup());
		data.getTextValue(UDA.LAUF_ATT_NAME).setText(getName());
		data.getTextValue(UDA.LAUF_ATT_BESCHREIBUNG).setText(getBeschreibung());
		final ClientSkript ref = getSkript();
		if (ref == null) {
			data.getReferenceValue(UDA.LAUF_ATT_SKRIPT).setSystemObject(null);
		} else {
			data.getReferenceValue(UDA.LAUF_ATT_SKRIPT).setSystemObject(ref.getDavObjekt());
		}

		final Data.Array historie = data.getArray(UDA.LAUF_ATT_HISTORIE);
		historie.setLength(1);
		historie.getItem(0).getTimeValue(UDA.ATL_HISTORIE_ATT_AENDERUNGSDATUM).setMillis(System.currentTimeMillis());
		historie.getItem(0).getTextValue(UDA.ATL_HISTORIE_ATT_NAME).setText(benutzer);
		historie.getItem(0).getTextValue(UDA.ATL_HISTORIE_ATT_GRUND).setText(grund);

		final Data.Array parameterArray = data.getArray(UDA.LAUF_ATT_PARAMETER);
		parameterArray.setLength(getParameter().keySet().size());
		int idx = 0;
		for (final String key : getParameter().keySet()) {
			parameterArray.getItem(idx).getTextValue("Name").setText(key);
			parameterArray.getItem(idx).getTextValue("Wert").setText(getParameter().get(key));
			idx++;
		}

		data.getTextValue(UDA.LAUF_ATT_BENUTZER).setText(getBenutzer());

		final Data.NumberArray passwort = data.getUnscaledArray(UDA.LAUF_ATT_PASSWORT);
		if (getPasswd().length() > 0) {
			final byte[] encrypted = modul.encrypt(getPasswd());
			passwort.setLength(encrypted.length);
			for (idx = 0; idx < encrypted.length; idx++) {
				passwort.getValue(idx).set(encrypted[idx]);
			}
		} else {
			passwort.setLength(0);
		}

		data.getUnscaledValue(UDA.LAUF_ATT_STARTTYP).set(getStartTyp().getCode());

		data.getItem(UDA.LAUF_ATT_STARTZYKLUS).getTimeValue(UDA.ATL_STARTZYKLUS_ATT_STARTZEITPUNKT)
		.setMillis(getZyklus().getStartZeitPunkt());
		data.getItem(UDA.LAUF_ATT_STARTZYKLUS).getUnscaledValue(UDA.ATL_STARTZYKLUS_ATT_INTERVALL)
		.set(getZyklus().getIntervall());

		final Data.Array protokollArray = data.getArray(UDA.LAUF_ATT_PROTOKOLL);
		final Collection<Protokoll> protokolle = getProtokolle();
		protokollArray.setLength(protokolle.size());
		idx = 0;
		for (final Protokoll protokoll : protokolle) {
			protokollArray.getItem(idx).getUnscaledValue(UDA.ATL_PROTOKOLL_ATT_TYP).set(protokoll.getTyp().getCode());
			protokollArray.getItem(idx).getTextValue(UDA.ATL_PROTOKOLL_ATT_NAME).setText(protokoll.getName());
			protokollArray.getItem(idx).getUnscaledValue(UDA.ATL_PROTOKOLL_ATT_ANZAHL).set(protokoll.getAnzahl());
			protokollArray.getItem(idx).getUnscaledValue(UDA.ATL_PROTOKOLL_ATT_MAXGROESSE).set(protokoll.getMaxSize());
			idx++;
		}

		return data;
	}

	/**
	 * liefert das Systemobjekt, das den Skriptlauf in der Konfiguration des
	 * Datenverteilers repräsentiert.
	 *
	 * @return das Systemobjekt
	 */
	public SystemObject getDavObjekt() {
		return objekt;
	}

	/**
	 * liefert das Ergebnis der letzten Skriptlaufausführung seit dem Start der
	 * "Umfassenden Datenanalyse".<br>
	 * Wurde bisher kein Skriptlauf ausgeführt, wird der Wert <i>null</i>
	 * geliefert.
	 *
	 * @return das Ergebnis oder <i>null</i>
	 */
	public SkriptLaufErgebnis getLaufErgebnis() {
		return letztesErgebnis;
	}

	/**
	 * liefert die Liste der Ausgabemeldungen.
	 *
	 * @return die Meldungen
	 */
	public List<String> getMeldungen() {
		return meldungen;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.uda.data.SkriptLauf#getModul()
	 */
	@Override
	public ClientModul getModul() {
		return modul;
	}

	/**
	 * liefert das dem Skriptlauf zugeordnete Skriptobjekt.
	 *
	 * @return das Skriptobjekt oder <i>null</i>, wenn keins zugewiesen ist
	 */
	@Override
	public ClientSkript getSkript() {
		return (ClientSkript) super.getSkript();
	}

	/**
	 * ermittelt, ob die Daten des Objekts gültig sind, d.h. ob sie von einer
	 * momentan aktiven Anwendung "Umfassende Datenanalyse" veröffentlicht
	 * wurden.
	 *
	 * @return valid
	 */
	public boolean isValid() {
		return valid;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.uda.data.SkriptLauf#publiziereErgebnis(java.lang.Exception)
	 */
	@Override
	public void publiziereErgebnis(final Exception fehler) {
		if (fehler != null) {
			ausgeben(new ProtokollEintrag(fehler.getMessage()));
		}
	}

	/**
	 * entfernt lokale Daten und Datenverteileranmeldungen des Skriptlaufs.
	 */
	void remove() {
		modul.getVerbindung().unsubscribeReceiver(this, objekt, istDesc);
		modul.getSender().abmelden(objekt, vorgabeDesc);
		modul.getSender().abmelden(objekt, kommandoDesc);
		modul.getVerbindung().unsubscribeReceiver(this, objekt, kommandoRespDesc);
		modul.getVerbindung().unsubscribeReceiver(this, objekt, statusDesc);
		modul.getVerbindung().unsubscribeReceiver(this, objekt, logDesc);
	}

	/**
	 * überträgt die Daten des Skriptlaufs als neue Vorgabe an die "Umfassende
	 * Datenanalyse".
	 *
	 * @param benutzer
	 *            der Benutzerm der eine Änderung vornimmt
	 * @param grund
	 *            der Grund für die Änderung
	 *
	 * @throws ClientException
	 *             die Übertragung konnte nicht erfolgreich ausgeführt werden
	 */
	public void save(final String benutzer, final String grund) throws ClientException {
		if (!isValid()) {
			throw new ClientException("Daten des Objekts sind potentiell ungültig");
		}
		try {
			final Data data = fuelleDatensatz(benutzer, grund);
			modul.getSender().warteAufBestaetigung(objekt, vorgabeDesc, 20000L);
			modul.getVerbindung().sendData(new ResultData(objekt, vorgabeDesc, System.currentTimeMillis(), data));
		} catch (final DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
			Debug.getLogger().warning(e.getLocalizedMessage(), e);
			// kann nicht auftreten, da vor dem Senden auf die
			// Anmeldebestätigung gewartet wird
		}
	}

	/**
	 * versenden eines Kommandos zum Verändern des Ausführungsstatus eines
	 * Skriptlaufs an die "Umfassende Datenanalyse".
	 *
	 * @param kommando
	 *            das auszuführende Kommando
	 * @return das Ergebnis der Übertragung
	 * @throws ClientException
	 *             das kommando an den Skriptlauf konnte nicht ordnungsgemäß
	 *             übertragen werden
	 */
	private LaufKommandoAntwort sendeKommando(final SkriptLaufKommando kommando) throws ClientException {
		final Data data = modul.getVerbindung().createData(kommandoDesc.getAttributeGroup());
		data.getUnscaledValue("Anwendung").set(kommandoAntwortBeobachter.getAppId());

		final long id = kommandoAntwortBeobachter.getId();

		data.getUnscaledValue("Id").set(id);
		data.getUnscaledValue("Kommando").set(kommando.getCode());
		data.getTextArray("Meldung").setLength(0);

		try {
			modul.getSender().warteAufBestaetigung(objekt, kommandoDesc, 20000L);
			modul.getVerbindung().sendData(new ResultData(objekt, kommandoDesc, System.currentTimeMillis(), data));
		} catch (final SendSubscriptionNotConfirmed e) {
			throw new ClientException("Kommando konnte nicht versendet werden",
					e);
		}

		return kommandoAntwortBeobachter.liesAntwort(id, 20000L);
	}

	/**
	 * Es wird ein Kommando zum Starten des Skriptlaufs an die "Umfassende
	 * Datenanalyse" übertragen.
	 *
	 * @return das Ergebnis des Kommandoausführung
	 *
	 * @throws ClientException
	 *             der Skriptlauf konnte nicht gestartet werden
	 */
	public LaufKommandoAntwort starten() throws ClientException {
		return sendeKommando(SkriptLaufKommando.STARTEN);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion liefert den Namen des Skriptlaufs.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getName();
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion aktualisiert die Daten des Objekts mit den aktuelllen von
	 * der "Umfassenden Datenanalyse" empfangenen.
	 *
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(final ResultData[] resultListe) {
		for (final ResultData result : resultListe) {
			final Data data = result.getData();
			final AttributeGroup atg = result.getDataDescription().getAttributeGroup();
			if (atg.equals(istDesc.getAttributeGroup())) {
				if (data != null) {
					this.aktualisiereDaten(data);
				} else {
					valid = false;
				}
				synchronized (getModul().getLaufListenerListe()) {
					for (final SkriptLaufListener listener : modul.getLaufListenerListe()) {
						listener.skriptAktualisiert(this);
					}
				}
			} else if (atg.equals(statusDesc.getAttributeGroup())) {
				if (data != null) {
					setStatus(SkriptLaufStatus.getZustand(data.getUnscaledValue("Status").intValue()));
				} else {
					setStatus(SkriptLaufStatus.PASSIV);
				}
				synchronized (modul.getLaufListenerListe()) {
					for (final SkriptLaufListener listener : modul.getLaufListenerListe()) {
						listener.skriptZustandAktualisiert(this);
					}
				}
			} else if (atg.equals(kommandoRespDesc.getAttributeGroup())) {
				kommandoAntwortBeobachter.antwort(data);
			} else if (atg.equals(logDesc.getAttributeGroup())) {
				meldungen.clear();
				if (data != null) {
					final Data.Array array = data.getArray("Meldung");
					for (int idx = 0; idx < array.getLength(); idx++) {
						meldungen.add(array.getTextValue(idx).getText());
					}
				}
				synchronized (modul.getLaufListenerListe()) {
					for (final SkriptLaufListener listener : modul.getLaufListenerListe()) {
						listener.meldungsListeAktualisiert(this);
					}
				}
			} else if (atg.equals(resultDesc.getAttributeGroup())) {
				if (data == null) {
					letztesErgebnis = null;
				} else {
					letztesErgebnis = new SkriptLaufErgebnis(result.getDataTime(),
							data.getUnscaledValue("Abgeschlossen").intValue() != 0);
					final Data.TextArray array = data.getTextArray("Meldung");
					for (int idx = 0; idx < array.getLength(); idx++) {
						letztesErgebnis.addMeldung(array.getText(idx));
					}
					synchronized (modul.getLaufListenerListe()) {
						for (final SkriptLaufListener listener : modul.getLaufListenerListe()) {
							listener.ergebnisAktualisiert(this);
						}
					}
				}
			}
		}
	}
}
