/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.client;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Repräsentation des Ergebnis eines Skriptlaufs.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SkriptLaufErgebnis {

	/**
	 * der Skriptlauf wurde vollständig abgeschlossen.
	 */
	private final boolean ok;

	/**
	 * der Zeitpunkt des Ergebnisses.
	 */
	private final long zeit;

	/**
	 * die Meldungen des Ergebnisses.
	 */
	private final List<String> meldungen = new ArrayList<>();

	/**
	 * Konstruktor.
	 *
	 * @param zeit
	 *            der Zeitpunkt der Ergenismeldung
	 * @param ok
	 *            Skript wurde vollständig ausgeführt ?
	 */
	public SkriptLaufErgebnis(final long zeit, final boolean ok) {
		this.ok = ok;
		this.zeit = zeit;
	}

	/**
	 * liefert den Meldungstext des Skripts als mit ';' getrennte Zeichenkette.
	 * <br>
	 * Es werden soviele zusätzliche Meldungen an die ursprüngliche Meldung
	 * angehangen, wie im Parameter anzahl vorgegeben wurde.
	 *
	 * @param anzahl
	 *            die Anzahl der zusätzlich angehangenen Meldungen
	 * @return der Text
	 */
	public String getText(final int anzahl) {
		final StringBuffer buffer = new StringBuffer();

		int idx = 0;
		for (final String meldung : meldungen) {
			if (buffer.length() > 0) {
				buffer.append(';');
			}
			buffer.append(meldung);
			idx++;
			if (idx >= anzahl) {
				break;
			}
		}

		return buffer.toString();
	}

	/**
	 * liefert die Meldungszeit als Zeichenkette im Standardformat.
	 *
	 * @return die Zeit als Zeichenkette
	 */
	public String getZeitString() {
		return DateFormat.getDateTimeInstance().format(new Date(zeit));
	}

	/**
	 * fügt dem Ergebnis eine Meldungszeile hinzu.
	 *
	 * @param text
	 *            die neue Zeile
	 */
	void addMeldung(final String text) {
		meldungen.add(text);
	}

	/**
	 * die Skriptlaufausführung wurde vollständig abgschlossen.
	 *
	 * @return true, wenn der Skriptlauf vollständig ausgeführt wurde.
	 */
	public boolean isOk() {
		return ok;
	}

	/**
	 * liefert die Liste der Meldungseinträge.
	 *
	 * @return die Liste
	 */
	public List<String> getMeldungen() {
		return meldungen;
	}

	/**
	 * liefert den Zeitpunkt der Rückmeldung in Millisekunden seit dem 1.1.1970
	 * 0Uhr GMT.
	 *
	 * @return den Zeitpunkt
	 */
	public long getZeit() {
		return zeit;
	}
}
