/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung.iteration;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.ausdruck.Variable;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Bedingung für eine Iteration mit einer numerischen Laufvariable.<br>
 * Die Verwendung entspricht einer FOR-Schleife mit einem numerischen Laufindex,
 * einem vorgegebenen Startwert und einer optionalen Schrittweite für die
 * Inkrementierung der Laufvariable.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class LaufIterationsBedingung implements IterationsBedingung {

	/**
	 * der Name der Laufvariable.
	 */
	private final String variable;

	/**
	 * der Ausdruck zur Beschreibung des Startwerts für die Laufvariable.
	 */
	private final Ausdruck von;

	/**
	 * der Ausdruck für die Beschreibung des Endwerts für die Laufvariable.
	 */
	private final Ausdruck bis;

	/**
	 * der Asudruck für die Beschreibung der Schrittweite für die Erhöhung des
	 * Wertes der Laufvariable.
	 */
	private final Ausdruck schrittweite;

	/**
	 * Markierung, ob die Laufvariable bereits initialisiert wurde.
	 */
	private boolean initialized;

	/**
	 * Konstruktor.
	 *
	 * @param variable
	 *            die Variable für den Laufindex
	 * @param von
	 *            der Ausdruck für den Startwert
	 * @param bis
	 *            der Ausdruck für den Endwert
	 * @param schrittweite
	 *            der Ausdruck für die Schrittweite
	 */
	public LaufIterationsBedingung(final Variable variable, final Ausdruck von, final Ausdruck bis,
			final Ausdruck schrittweite) {
		this.variable = variable.getName();
		this.von = von;
		this.bis = bis;
		this.schrittweite = schrittweite;
	}

	/**
	 * Die Funktion setzt den Wert der Laufvariable. Dazu wird im ersten Schritt
	 * der Ausdruck für den Startwert ausgewertet. In jedem folgenden Schritt
	 * der Ausdruck für die Schrittweite. Der Inhalt der Laufvariable wird mit
	 * dem Wert der Auswertung des Ausdrucks für den Endwert verglichen.<br>
	 * Es wird der Wert "wahr" geliefert solange der Wert der Laufvariable &lt;=
	 * dem Wert des Ausdrucks für den Endwert ist.<br>
	 * Nicht behandelte Fehler bei der Auswertung der Ausdrücke werden
	 * weitergereicht.
	 *
	 * @param kontext
	 *            der Kontext, in dem der Ausdruck ausgewertet wird
	 * @return einen {@link LogischerWert}, der den Wert der Bedingung darstellt
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public LogischerWert interpret(final Kontext kontext) {
		boolean result = true;
		if (initialized) {
			long offset = 1;
			if (schrittweite != null) {
				offset = ((Number) schrittweite.interpret(kontext)).longValue();
			}
			final Number startWert = (Number) kontext.get(variable);
			if (startWert != null) {
				kontext.set(variable, Long.valueOf(startWert.longValue() + offset));
			}
		} else {
			kontext.set(variable, von.interpret(kontext));
			initialized = true;
		}
		final Number aktuellerWert = (Number) kontext.get(variable);
		final Object bisObjekt = bis.interpret(kontext);
		if (bisObjekt instanceof Number) {
			final Number endWert = (Number) bisObjekt;
			if ((aktuellerWert == null) || (aktuellerWert.longValue() > endWert.longValue())) {
				result = false;
				initialized = false;
			}
		} else {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
		}

		return LogischerWert.of(result);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#getLaufVariable()
	 */
	@Override
	public String getLaufVariable() {
		return variable;
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#isGueltig(de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext)
	 */
	@Override
	public boolean isGueltig(final SkriptKontext kontext) {
		return interpret(kontext).get();
	}

	/**
	 * {@inheritDoc}. Die Markierung für die Initialisierung der Laufvariable
	 * wird zurückgesetzt.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#reset()
	 */
	@Override
	public void reset() {
		initialized = false;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(new Variable(variable));
		liste.add(von);
		liste.add(bis);
		liste.add(schrittweite);
		return liste;
	}
}
