/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung.iteration;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.ausdruck.Variable;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer;
import de.bsvrz.ibv.uda.interpreter.daten.container.UdaIterator;
import de.bsvrz.ibv.uda.interpreter.daten.dav.MengenObjekt;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Iterationsbedingung für die Iteration über eine Menge.<br>
 * Die Bedingung wird für Iterationen verwendet, in denen jedes Objekt aus einer
 * übergebenen Menge bewertet wird.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class MengenIterationsBedingung implements IterationsBedingung {

	/**
	 * der Name der Laufvariable.
	 */
	private final String variable;

	/**
	 * der Ausdruck für die Beschreibung der Menge über die iteriert werden
	 * soll.
	 */
	private final Ausdruck menge;

	/**
	 * der Iterator für den Durchlauf über die Menge.
	 */
	private UdaIterator iterator;

	/**
	 * Konstruktor.
	 *
	 * @param variable
	 *            die Variable für den Zugriff auf die Elemente der Menge
	 * @param menge
	 *            der Ausdruck für die Beschreibung der Menge
	 */
	public MengenIterationsBedingung(final Variable variable, final Ausdruck menge) {
		this.variable = variable.getName();
		this.menge = menge;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#getLaufVariable()
	 */
	@Override
	public String getLaufVariable() {
		return variable;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(new Variable(variable));
		liste.add(menge);
		return liste;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion wertet im ersten Schritt den Ausdruck für die Beschreibung
	 * der Menge aus. Die Auswertung des Ausdrucks muss einen Container liefern
	 * für den ein Iterator gebildet wird. Gültige Containerobjekte sind:
	 * <ul>
	 * <li>Felder</li>
	 * <li>Listen</li>
	 * <li>Schlüsselmengen</li>
	 * <li>Mengenobjekte (ObjectSet aus der Abfrag der
	 * Datenverteilerkonfiguration)</li>
	 * </ul>
	 * In den folgenden Schritten wird jeweils nur der Iterator vorwärts bewegt.
	 * <br>
	 * Das Objekt auf das der Iterator zeigt wird jeweils an die definierte
	 * Variable übergeben. Die Funktion liefert solange den Wert "wahr" bis der
	 * Iterator nicht mehr auf ein gültiges Element plaziert ist.
	 *
	 * @param kontext
	 *            der Kontext, in dem der Ausdruck ausgewertet wird
	 * @return einen {@link LogischerWert}, der den Wert der Bedingung darstellt
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public LogischerWert interpret(final Kontext kontext) {
		boolean result = false;
		if (iterator == null) {
			final Object container = menge.interpret(kontext);
			if (container instanceof UdaContainer) {
				iterator = ((UdaContainer) container).getUdaIterator(0);
			} else if (container instanceof MengenObjekt) {
				iterator = ((UdaContainer) ((MengenObjekt) container).getStrukturElement("objekte")).getUdaIterator(0);
			}
		}
		if (iterator != null) {
			iterator.weiter();
			if (iterator.innerhalb().get()) {
				kontext.set(variable, iterator.getElement());
				result = true;
			} else {
				iterator = null;
			}
		}
		return LogischerWert.of(result);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#isGueltig(de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext)
	 */
	@Override
	public boolean isGueltig(final SkriptKontext kontext) {
		return interpret(kontext).get();
	}

	/**
	 * {@inheritDoc}.<br>
	 * Der Iterator für das Durchlaufen der Menge wird entfernt.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#reset()
	 */
	@Override
	public void reset() {
		iterator = null;
	}
}
