/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.uda;

import java.io.File;
import java.util.Set;
import java.util.Timer;
import java.util.TreeSet;

import de.bsvrz.dav.daf.main.ApplicationCloseActionHandler;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.ibv.uda.uda.data.UDA;
import de.bsvrz.ibv.uda.verwaltung.UdaVerwaltung;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList.Argument;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * die Applikation "Umfassende Datenanalyse"<br>
 * Die Klasse erweitert die Datenverteiler-<i>StandardApplication</i> , d.h. es
 * wird eine Datenverteilerverbindung hergestellt. Die Anwendung stellt einen
 * Server zur Verwaltung und Ausführung der für eine UdaModul definierten
 * Skripte mit den zugehörigen Skriptläufen dar.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UmfassendeDatenanalyse implements StandardApplication, ApplicationCloseActionHandler {

	/**
	 * der Parametername zur Angabe des zu verwaltenden UdaModuls.
	 */
	private static final String START_PARAMETER_UDA_MODUL = "-konfiguration";

	/**
	 * der alternative Parametername zur Angabe des zu verwaltenden UdaModuls.
	 */
	private static final String START_PARAMETER_UDA_MODUL_ALTERNATIVE = "-udaModul";

	/**
	 * der Parametername zur Angabe des Wurzelverzeichnisses zum Ablegen von
	 * Daten für die "Umfassende Datenanalyse" (Skripte und
	 * Skriptluafdefinitionen).
	 */
	private static final String START_PARAMETER_UDA_ROOT = "-udaVerzeichnis";

	/**
	 * der alternative Parametername zur Angabe des Wurzelverzeichnisses zum
	 * Ablegen von Daten für die "Umfassende Datenanalyse" (Skripte und
	 * Skriptluafdefinitionen).
	 */
	private static final String START_PARAMETER_UDA_ROOT_ALTERNATIVE = "-udaRoot";

	/**
	 * der Name des Wurzelverzeichnisses, in dem die Uda-Daten auf dem Server
	 * abgelegt werden.
	 */
	private static File udaModulRoot = new File("udaDaten");

	/**
	 * die Startfunktion für die Anwendung "Umfassende Datenanalyse".
	 *
	 * @param args
	 *            die an die Applikation übergebenen Argumente
	 */
	public static void main(final String[] args) {
		StandardApplicationRunner.run(new UmfassendeDatenanalyse(), args);
	}

	/**
	 * die Datenverteilerverbindung.
	 */
	private ClientDavInterface verbindung;

	/**
	 * die Liste der PID für die zu verwaltenden UdaModule.
	 */
	private final Set<String> udaModulPids = new TreeSet<>();

	/**
	 * der Timer zur Realisierung des zeitgesteuerten Starts von Skriptläufen.
	 */
	private final Timer startTimer = new Timer();

	/** Logger für Debugausgaben. */
	private Debug logger;

	/**
	 * beendet den Timerthread beim Abbruch der Datenverteilerverbindung.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.dav.daf.main.ApplicationCloseActionHandler#close(java.lang.String)
	 */
	@Override
	public void close(final String error) {
		startTimer.cancel();
		System.exit(-1);
	}

	/**
	 * liefert den globalen Timer der Anwendung für das zyklische und
	 * zeitgesteuerte Ausführen von Skripten bzw, Skriptläufen.
	 *
	 * @return startTimer
	 */
	public Timer getStartTimer() {
		return startTimer;
	}

	/**
	 * liefert den Namen des Wurzelverzeichnisses für die Speicherung der
	 * Uda-Daten.
	 *
	 * @return den Namen
	 */
	public File getUdaModulRoot() {
		return UmfassendeDatenanalyse.udaModulRoot;
	}

	/**
	 * liefert die Datenverteilerverbindung der Anwendung.
	 *
	 * @return die Verbindung
	 */
	public ClientDavInterface getVerbindung() {
		return verbindung;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion initialisiert die allgemeinen Parameter der Umfassenden
	 * Datenanalyse und ermittelt die PID der ui verwaltenden UdaModule. Danach
	 * wir das Verwaltungsmodul gestartet.
	 *
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	@SuppressWarnings("unused")
	@Override
	public void initialize(final ClientDavInterface davIf) throws Exception {
		verbindung = davIf;

		// Initialisiern der Betriebmeldungsverwaltung
		// MessageSender.getInstance().init(davIf, "Umfassende Datenanalyse",
		// "UDA" + System.currentTimeMillis());

		if (!UmfassendeDatenanalyse.udaModulRoot.exists()) {
			UmfassendeDatenanalyse.udaModulRoot.mkdirs();
		}

		if (udaModulPids.isEmpty()) {

			logger.info("Keine PIDs für UDA-Module angegeben");

			final DataModel model = verbindung.getDataModel();
			final SystemObjectType udaModulTyp = model.getType(UDA.TYP_MODUL);
			if (udaModulTyp != null) {
				for (final SystemObject obj : udaModulTyp.getObjects()) {
					logger.info("Gefundenes UdaModul: " + obj);
					udaModulPids.add(obj.getPid());
				}
			} else {
				LogTools.log(logger, UdaMessage.ErrorUdaModulTyp);
				throw new IllegalStateException(UdaMessage.ErrorUdaModulTyp.toString());
			}
		}

		verbindung.setCloseHandler(this);

		new UdaVerwaltung(this, udaModulPids);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Folgende Argumente werden neben den Argumenten für eine
	 * Datenverteiler-Standardapplikation ausgewertet:
	 * <ul>
	 * <li><b>udaModul:</b> die PID der zu verwaltenden UdaModule.</li>
	 * <li><b>udaRoot:</b> das Verzeichnis zur Speicherung der Uda-Daten auf dem
	 * Server.</li>
	 * </ul>
	 *
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#parseArguments(de.bsvrz.sys.funclib.commandLineArgs.ArgumentList)
	 */
	@Override
	public void parseArguments(final ArgumentList argList) throws Exception {

		logger = Debug.getLogger();

		while (argList.hasUnusedArguments()) {
			final Argument arg = argList.fetchNextArgument();
			if (UmfassendeDatenanalyse.START_PARAMETER_UDA_MODUL.equals(arg.getName())
					|| UmfassendeDatenanalyse.START_PARAMETER_UDA_MODUL_ALTERNATIVE.equals(arg.getName())) {
				final String pid = arg.getValue();
				if ((pid != null) && (pid.length() > 0)) {
					udaModulPids.add(arg.getValue());
				}
			} else if (UmfassendeDatenanalyse.START_PARAMETER_UDA_ROOT.equals(arg.getName())
					|| UmfassendeDatenanalyse.START_PARAMETER_UDA_ROOT_ALTERNATIVE.equals(arg.getName())) {
				UmfassendeDatenanalyse.udaModulRoot = new File(arg.getValue());
			}
		}
	}
}
